package com.persagy.dmp.admin.config.shiro;

import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.admin.entity.AdminUser;
import com.persagy.dmp.admin.entity.QAdminUser;
import com.persagy.dmp.admin.repository.AdminPermissionRepository;
import com.persagy.dmp.admin.repository.AdminUserRepository;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authc.*;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.authz.SimpleAuthorizationInfo;
import org.apache.shiro.realm.AuthorizingRealm;
import org.apache.shiro.subject.PrincipalCollection;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Optional;

@Slf4j
@Component
public class DmpRealm extends AuthorizingRealm {

	@Autowired
	private AdminUserRepository userRepository;

	@Autowired
	private AdminPermissionRepository permissionRepository;

	/**
	 * 校验身份信息
	 *
	 * @param authenticationToken
	 * @return
	 * @throws AuthenticationException
	 */
	@Override
	protected AuthenticationInfo doGetAuthenticationInfo(AuthenticationToken authenticationToken) throws AuthenticationException {
		UsernamePasswordToken token = (UsernamePasswordToken) authenticationToken;
		String ssoToken = new String(token.getPassword());
		log.info("身份认证 --> {} {}", token.getUsername(), ssoToken);
		Optional<AdminUser> one = userRepository.findOne(QAdminUser.adminUser.username.eq(token.getUsername()));
		if (!one.isPresent()) {
			return null;
		}
		AdminUser user = one.get();

		UserModel userModel = new UserModel();
		userModel.setId(user.getId());
		userModel.setUsername(user.getUsername());
		userModel.setToken(ssoToken);
		List<String> permissions = permissionRepository.queryForUser(user.getId());
		userModel.setPermissions(permissions);
		log.info("身份认证 --> {}", JacksonMapper.toFormatJson(userModel));
		SimpleAuthenticationInfo result = new SimpleAuthenticationInfo(userModel, ssoToken, "dmpRealm");
		return result;
	}

	/**
	 * 校验权限
	 *
	 * @param principals
	 * @return
	 */
	@Override
	protected AuthorizationInfo doGetAuthorizationInfo(PrincipalCollection principals) {
		UserModel userModel = (UserModel) principals.getPrimaryPrincipal();
		log.info("权限配置--> {} {}", userModel.getUsername(), JacksonMapper.toSimpleJson(userModel.getPermissions()));
		SimpleAuthorizationInfo authorizationInfo = new SimpleAuthorizationInfo();
		authorizationInfo.addStringPermissions(userModel.getPermissions());
		return authorizationInfo;
	}

}

