package com.persagy.dmp.rwd.edit.service;

import com.persagy.common.criteria.CriteriaExpressionParser;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.ListResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.rwd.edit.entity.QFuncidDef;
import com.persagy.dmp.rwd.edit.util.ClassCodeUtils;
import com.persagy.dmp.rwd.model.FuncidDefModel;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@ExtendWith(MockitoExtension.class)
class FuncidDefServiceTest {

    @Mock
    private ClassDefService mockClassDefService;
    @Mock
    private ClassCodeUtils mockClassCodeUtils;
    @Mock
    private CriteriaUtils mockCriteriaUtils;

    @InjectMocks
    private FuncidDefService funcidDefServiceUnderTest;

    @Test
    void testQueryFuncid() {
        // Setup
        final JacksonCriteria request = JacksonCriteria.newInstance();
        when(mockClassDefService.queryClass(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());
        when(mockCriteriaUtils.query(eq(new QFuncidDef("variable")), any(CriteriaExpressionParser.class), any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());

        // Run the test
        final ListResponse<FuncidDefModel> result = funcidDefServiceUnderTest.queryFuncid(request);

        // Verify the results
        verify(mockClassCodeUtils).prepareCriteriaForFuncid(any(JacksonCriteria.class), eq("codeName"));
    }

    @Test
    void testQuery() {
        // Setup
        final JacksonCriteria criteria = JacksonCriteria.newInstance();
        when(mockCriteriaUtils.query(eq(new QFuncidDef("variable")), any(CriteriaExpressionParser.class), any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());

        // Run the test
        final PagedResponse<FuncidDefModel> result = funcidDefServiceUnderTest.query(criteria);

        // Verify the results
        verify(mockClassCodeUtils).prepareCriteriaForFuncid(any(JacksonCriteria.class), eq("codeName"));
    }
}
